using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using KellermanSoftware.NetEncryptionLibrary;

namespace NetEncryptionLibraryDemo
{
    public partial class Demo : Form
    {
        // The 'target' is used to point to the textbox that the open/save
        //  dialog is currently associated with.
        private System.Windows.Forms.TextBox target;

        // One encryption object is used to perform all operations
        private Encryption encrypt;

        public Demo()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Show an error message box
        /// </summary>
        /// <param name="myException"></param>
        private void ShowError(Exception myException)
        {
            Hourglass(false);
            MessageBox.Show(myException.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            try
            {
                // Create the encryption object and start off with a randomly-generated
                //  initialization vector.
                encrypt = new Encryption();
                txtEncryptionIV.Text = encrypt.InitializationVector;
            }
            catch (Exception myException)
            {
                ShowError(myException);
                Application.Exit();
            }
        }

        private void openFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            // Update the associated text box
            target.Text = openFileDialog.FileName;
        }

        private void saveFileDialog_FileOk(object sender, CancelEventArgs e)
        {
            
            // Update the associated text box
            target.Text = saveFileDialog.FileName;
        }

        private void btnBrowseCompressionInput_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtCompressionInput;
            openFileDialog.ShowDialog();
        }

        private void btnBrowseCompressionOutput_Click(object sender, EventArgs e)
        {
            // Show the save file dialog, associated with this text box
            target = txtCompressionOutput;
            saveFileDialog.ShowDialog();
        }

        private void btnBrowseFileErase_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtFileErase;
            openFileDialog.ShowDialog();
        }

        private void btnBrowseDirectoryErase_Click(object sender, EventArgs e)
        {
            // Save the results of the browse dialog into this text box
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
              txtDirectoryErase.Text = folderBrowserDialog.SelectedPath;
        }

        private void btnBrowseMultipleErase_Click(object sender, EventArgs e)
        {
            // Save the results of the browse dialog into this text box
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
                txtMultipleErase.Text = folderBrowserDialog.SelectedPath;
        }

        private void btnFileErase_Click(object sender, EventArgs e)
        {
            // Check to make sure the user selected a file
            if (txtFileErase.Text == "")
            {
                MessageBox.Show("Please select a file to erase");
                return;
            }

            try
            {
                Hourglass(true);
                // Securely erase the file
                encrypt.SecureFileErase(txtFileErase.Text);

                Hourglass(false);
                MessageBox.Show("Finished");
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnDirectoryErase_Click(object sender, EventArgs e)
        {
            // Check to make sure the user selected a directory
            if (txtDirectoryErase.Text == "")
            {
                MessageBox.Show("Please select a directory to erase");
                return;
            }

            try
            {
                Hourglass(true);

                // Securely erase the directory
                encrypt.SecureDirectoryErase(txtDirectoryErase.Text);

                Hourglass(false);
                MessageBox.Show("Finished");

            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnMultipleErase_Click(object sender, EventArgs e)
        {
            // Check to make sure the user selected a directory and file mask
            if (txtMultipleErase.Text == "")
            {
                MessageBox.Show("Please select a directory to erase");
                return;
            }
            if (txtMultipleEraseMask.Text == "")
            {
                MessageBox.Show("Please type a mask for deleted files");
                return;
            }

            Hourglass(true);

            // If the Erase Subdirs checkbox is checked, we want to erase files in
            //  all subdirectories as well.
            SearchOption option = new SearchOption();
            if (cbxMultipleEraseSubdirs.Checked)
                option = SearchOption.AllDirectories;
            else
                option = SearchOption.TopDirectoryOnly;

            try
            {
                // Securely erase the files in the selected directory
                encrypt.SecureDirectoryErase(txtMultipleErase.Text, txtMultipleEraseMask.Text, option);

                Hourglass(false);
                MessageBox.Show("Finished");

            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnCompress_Click(object sender, EventArgs e)
        {
            try
            {
                // Check to make sure the user selected input and output files
                if (txtCompressionInput.Text == "")
                {
                    MessageBox.Show("Please select a file to compress");
                    return;
                }
                if (txtCompressionOutput.Text == "")
                {
                    MessageBox.Show("Please select a destination for the compressed file");
                    return;
                }

                Hourglass(true);

                // Do the actual compression
                switch (cbxCompressionType.Text)
                {
                    case "Deflate":
                        encrypt.CompressFile(CompressionType.Deflate, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                    case "GZip":
                        encrypt.CompressFile(CompressionType.GZip, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                }

                // Move the output file selection to the input file and clear the output file
                //  (so the user just has to choose an ouput file and then decompress)
                txtCompressionInput.Text = txtCompressionOutput.Text;
                txtCompressionOutput.Text = "";

                Hourglass(false);
                MessageBox.Show("Finished");
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnDecompress_Click(object sender, EventArgs e)
        {
            try
            {
                // Check to make sure the user selected input and output files
                if (txtCompressionInput.Text == "")
                {
                    MessageBox.Show("Please select a file to decompress");
                    return;
                }
                if (txtCompressionOutput.Text == "")
                {
                    MessageBox.Show("Please select a destination for the decompressed file");
                    return;
                }

                Hourglass(true);

                // Do the actual decompression
                switch (cbxCompressionType.Text)
                {
                    case "Deflate":
                        encrypt.DecompressFile(CompressionType.Deflate, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                    case "GZip":
                        encrypt.DecompressFile(CompressionType.GZip, txtCompressionInput.Text, txtCompressionOutput.Text);
                        break;
                }

                // Move the output file selection to the input file and clear the output file
                //  (so the user just has to choose an ouput file and then compress)
                txtCompressionInput.Text = txtCompressionOutput.Text;
                txtCompressionOutput.Text = "";

                Hourglass(false);
                MessageBox.Show("Finished");

            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnUUEncode_Click(object sender, EventArgs e)
        {
            try
            {
                // Do the uuEncoding, and display the result
                txtUUEncoded.Text = encrypt.uuEncode(txtUUPlaintext.Text);
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }

        }

        private void btnUUDecode_Click(object sender, EventArgs e)
        {
            try
            {
                // Do the uuDecoding, and display the result
                txtUUPlaintext.Text = encrypt.uuDecode(txtUUEncoded.Text);
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        private void btnEncrypt_Click(object sender, EventArgs e)
        {
            try
            {
                // Grab the IV and salt parameters
                encrypt.InitializationVector = txtEncryptionIV.Text;
                encrypt.Salt = txtEncryptionSalt.Text;

                // Do the actual encryption and decryption, based on the algorithm specified
                switch (cbxEncryptionType.Text)
                {
                    case "Rijndael (AES)":
                        txtEncryptionEncryptedString.Text = encrypt.EncryptString(EncryptionProvider.Rijndael, txtEncryptionKey.Text, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = encrypt.DecryptString(EncryptionProvider.Rijndael, txtEncryptionKey.Text, txtEncryptionEncryptedString.Text);
                        break;
                    case "DES":
                        txtEncryptionEncryptedString.Text = encrypt.EncryptString(EncryptionProvider.DES, txtEncryptionKey.Text, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = encrypt.DecryptString(EncryptionProvider.DES, txtEncryptionKey.Text, txtEncryptionEncryptedString.Text);
                        break;
                    case "RC2":
                        txtEncryptionEncryptedString.Text = encrypt.EncryptString(EncryptionProvider.RC2, txtEncryptionKey.Text, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = encrypt.DecryptString(EncryptionProvider.RC2, txtEncryptionKey.Text, txtEncryptionEncryptedString.Text);
                        break;
                    case "TripleDES":
                        txtEncryptionEncryptedString.Text = encrypt.EncryptString(EncryptionProvider.TripleDES, txtEncryptionKey.Text, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = encrypt.DecryptString(EncryptionProvider.TripleDES, txtEncryptionKey.Text, txtEncryptionEncryptedString.Text);
                        break;
                    case "Blowfish":
                        txtEncryptionEncryptedString.Text = encrypt.EncryptString(EncryptionProvider.Blowfish, txtEncryptionKey.Text, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = encrypt.DecryptString(EncryptionProvider.Blowfish, txtEncryptionKey.Text, txtEncryptionEncryptedString.Text);
                        break;
                    case "Twofish":
                        txtEncryptionEncryptedString.Text = encrypt.EncryptString(EncryptionProvider.Twofish, txtEncryptionKey.Text, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = encrypt.DecryptString(EncryptionProvider.Twofish, txtEncryptionKey.Text, txtEncryptionEncryptedString.Text);
                        break;
                    case "SHA1":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.SHA1, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "SHA256":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.SHA256, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "SHA384":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.SHA384, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "SHA512":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.SHA512, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "MD5":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.MD5, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "HAVAL":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.HAVAL, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "Whirlpool":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.Whirlpool, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "MD2":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.MD2, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "MD4":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.MD4, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "Tiger":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.Tiger, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "FCS16":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.FCS16, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "FCS32":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.FCS32, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;

                    case "Ripemd320":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.Ripemd320, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                    case "GostHash":
                        txtEncryptionEncryptedString.Text = encrypt.HashString(HashProvider.GostHash, txtEncryptionString.Text);
                        txtEncryptionDecryptedString.Text = "One way hash, no decryption possible.";
                        break;
                }
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }

        }

        private void btnBrowseFileChecksum_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtFileChecksum;
            openFileDialog.ShowDialog();

            // Check to make sure the user selected a file
            if (txtFileChecksum.Text == "")
            {
                return;
            }

            try
            {
                txtCRC32.Text = encrypt.GetFileCRC32(txtFileChecksum.Text).ToString();
                txtAdler32.Text = encrypt.GetFileAdler32(txtFileChecksum.Text).ToString();
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }

        /// <summary>
        /// Turn on/off hourglass cursor
        /// </summary>
        /// <param name="enable"></param>
        public void Hourglass(bool enable)
        {
            if (enable)
            {
                Cursor.Current = Cursors.WaitCursor;
                Application.DoEvents();

            }
            else
            {
                Cursor.Current = Cursors.Default;
                Application.DoEvents();
            }
        }

        private void btnEncryptFile_Click(object sender, EventArgs e)
        {
            // Show the open file dialog, associated with this text box
            target = txtFileToEncrypt;
            openFileDialog.ShowDialog();

            // Check to make sure the user selected a file
            if (txtFileToEncrypt.Text == "")
            {
                return;
            }

            try
            {
                string dir = Path.GetDirectoryName(txtFileToEncrypt.Text);
                string ext = Path.GetExtension(txtFileToEncrypt.Text);

                txtEncryptedFile.Text = Path.Combine(dir,"encrypted" + ext);
                txtDecryptedFile.Text = Path.Combine(dir,"decrypted" + ext);

                encrypt.EncryptFile(EncryptionProvider.Rijndael, "secret", txtFileToEncrypt.Text, txtEncryptedFile.Text);
                encrypt.DecryptFile(EncryptionProvider.Rijndael, "secret", txtEncryptedFile.Text, txtDecryptedFile.Text);
            }
            catch (Exception myException)
            {
                ShowError(myException);
            }
        }




    }
}